${CXX} $RECIPE_DIR/test.cpp -I$PREFIX/include  -L$PREFIX/lib -o test
[ $(./test) != "$PKG_VERSION" ] && exit 1

[ $($PYTHON -c 'import cv2; print(cv2.__version__)') != "$PKG_VERSION" ] && exit 1


set -ex



test -f $PREFIX/lib/libopencv_aruco.so
test -f $PREFIX/lib/libopencv_bgsegm.so
test -f $PREFIX/lib/libopencv_calib3d.so
test -f $PREFIX/lib/libopencv_ccalib.so
test -f $PREFIX/lib/libopencv_core.so
test -f $PREFIX/lib/libopencv_datasets.so
test -f $PREFIX/lib/libopencv_dnn.so
test -f $PREFIX/lib/libopencv_dpm.so
test -f $PREFIX/lib/libopencv_face.so
test -f $PREFIX/lib/libopencv_features2d.so
test -f $PREFIX/lib/libopencv_flann.so
test -f $PREFIX/lib/libopencv_fuzzy.so
test -f $PREFIX/lib/libopencv_highgui.so
test -f $PREFIX/lib/libopencv_imgcodecs.so
test -f $PREFIX/lib/libopencv_imgproc.so
test -f $PREFIX/lib/libopencv_line_descriptor.so
test -f $PREFIX/lib/libopencv_ml.so
test -f $PREFIX/lib/libopencv_objdetect.so
test -f $PREFIX/lib/libopencv_optflow.so
test -f $PREFIX/lib/libopencv_phase_unwrapping.so
test -f $PREFIX/lib/libopencv_photo.so
test -f $PREFIX/lib/libopencv_plot.so
test -f $PREFIX/lib/libopencv_reg.so
test -f $PREFIX/lib/libopencv_rgbd.so
test -f $PREFIX/lib/libopencv_saliency.so
test -f $PREFIX/lib/libopencv_shape.so
test -f $PREFIX/lib/libopencv_stereo.so
test -f $PREFIX/lib/libopencv_stitching.so
test -f $PREFIX/lib/libopencv_structured_light.so
test -f $PREFIX/lib/libopencv_superres.so
test -f $PREFIX/lib/libopencv_surface_matching.so
test -f $PREFIX/lib/libopencv_text.so
test -f $PREFIX/lib/libopencv_tracking.so
test -f $PREFIX/lib/libopencv_video.so
test -f $PREFIX/lib/libopencv_videoio.so
test -f $PREFIX/lib/libopencv_videostab.so
test -f $PREFIX/lib/libopencv_xfeatures2d.so
test -f $PREFIX/lib/libopencv_ximgproc.so
test -f $PREFIX/lib/libopencv_xobjdetect.so
test -f $PREFIX/lib/libopencv_xphoto.so
conda inspect linkages -p $PREFIX opencv
exit 0
